from typing import TypedDict, Optional, Union


class LeagueCategory(TypedDict):
    id: str
    current: Optional[bool]


class LeagueRule(TypedDict):
    id: str
    name: str
    description: Optional[str]


class League(TypedDict):
    id: str
    realm: Optional[str]
    description: Optional[str]
    category: Optional[LeagueCategory]
    rules: Optional[list[LeagueRule]]
    registerAt: Optional[str]
    event: Optional[bool]
    url: Optional[str]
    startAt: Optional[str]
    endAt: Optional[str]
    timedEvent: Optional[bool]
    scoreEvent: Optional[bool]
    delveEvent: Optional[bool]
    ancestorEvent: Optional[bool]
    leagueEvent: Optional[bool]


class Depth(TypedDict):
    default: Optional[int]
    solo: Optional[int]


class Character(TypedDict):
    id: str
    name: str
    level: int
    class_: str
    time: Optional[int]
    score: Optional[int]
    progress: Optional[dict]
    experience: Optional[int]
    depth: Optional[Depth]


class LadderEntry(TypedDict):
    rank: int
    dead: Optional[bool]
    retired: Optional[bool]
    ineligible: Optional[bool]
    public: Optional[bool]
    character: Character
    account: Optional['Account']


class PrivateLeague(TypedDict):
    name: str
    url: str


class EventLadderEntry(TypedDict):
    rank: int
    ineligible: Optional[bool]
    time: Optional[int]
    private_league: PrivateLeague


class Guild(TypedDict):
    id: int
    name: str
    tag: str


class TwitchStream(TypedDict):
    name: str
    image: str
    status: str


class Twitch(TypedDict):
    name: str
    stream: Optional[TwitchStream]


class Challenges(TypedDict):
    set: str
    completed: int
    max: int


class Account(TypedDict):
    name: str
    realm: Optional[str]
    guild: Optional[Guild]
    challenges: Optional[Challenges]
    twitch: Optional[Twitch]


class PvPMatch(TypedDict):
    id: str
    realm: Optional[str]
    startAt: Optional[str]
    endAt: Optional[str]
    url: Optional[str]
    description: str
    glickoRatings: bool
    pvp: bool
    style: str
    registerAt: Optional[str]
    complete: Optional[bool]
    upcoming: Optional[bool]
    inProgress: Optional[bool]


class PvPLadderTeamMember(TypedDict):
    account: Account
    character: Character
    public: Optional[bool]


class PvPLadderTeamEntry(TypedDict):
    rank: int
    rating: Optional[int]
    points: Optional[int]
    games_played: Optional[int]
    cumulative_opponent_points: Optional[int]
    last_game_time: Optional[str]
    members: list[PvPLadderTeamMember]


class ItemSocket(TypedDict):
    group: int
    attr: Optional[str]
    sColour: Optional[str]


class ItemProperty(TypedDict):
    name: str
    values: list[list[Union[str, int]]]
    displayMode: Optional[int]
    progress: Optional[float]
    type: Optional[int]
    suffix: Optional[str]


class LogbookFaction(TypedDict):
    id: str
    name: str


class LogbookMod(TypedDict):
    name: str
    faction: LogbookFaction


class ItemReward(TypedDict):
    label: str
    rewards: dict[str, int]


class IncubatedItem(TypedDict):
    name: str
    level: int
    progress: int
    total: int


class Crucible(TypedDict):
    layout: str
    nodes: dict[str, 'CrucibleNode']


class Scourged(TypedDict):
    tier: int
    level: Optional[int]
    progress: Optional[int]
    total: Optional[int]


class UltimatumMod(TypedDict):
    type: str
    tier: int


class Hybrid(TypedDict):
    isVaalGem: Optional[bool]
    baseTypeName: str
    properties: Optional[list[ItemProperty]]
    explicitMods: Optional[list[str]]
    secDescrText: Optional[str]


class Extended(TypedDict):
    category: Optional[str]
    subcategories: Optional[list[str]]
    prefixes: Optional[int]
    suffixes: Optional[int]


class Item(TypedDict):
    verified: bool
    w: int
    h: int
    icon: str
    support: Optional[bool]
    stackSize: Optional[int]
    maxStackSize: Optional[int]
    stackSizeText: Optional[str]
    league: Optional[str]
    id: Optional[str]
    influences: Optional[dict]
    elder: Optional[bool]
    shaper: Optional[bool]
    searing: Optional[bool]
    tangled: Optional[bool]
    abyssJewel: Optional[bool]
    delve: Optional[bool]
    fractured: Optional[bool]
    synthesised: Optional[bool]
    sockets: Optional[list[ItemSocket]]
    socketedItems: Optional[list['Item']]
    name: str
    typeLine: str
    baseType: str
    rarity: Optional[str]
    identified: bool
    itemLevel: Optional[int]
    ilvl: int
    note: Optional[str]
    forum_note: Optional[str]
    lockedToCharacter: Optional[bool]
    lockedToAccount: Optional[bool]
    duplicated: Optional[bool]
    split: Optional[bool]
    corrupted: Optional[bool]
    unmodifiable: Optional[bool]
    cisRaceReward: Optional[bool]
    seaRaceReward: Optional[bool]
    thRaceReward: Optional[bool]
    properties: Optional[list[ItemProperty]]
    notableProperties: Optional[list[ItemProperty]]
    requirements: Optional[list[ItemProperty]]
    additionalProperties: Optional[list[ItemProperty]]
    nextLevelRequirements: Optional[list[ItemProperty]]
    talismanTier: Optional[int]
    rewards: Optional[list[ItemReward]]
    secDescrText: Optional[str]
    utilityMods: Optional[list[str]]
    logbookMods: Optional[list[LogbookMod]]
    enchantMods: Optional[list[str]]
    scourgeMods: Optional[list[str]]
    implicitMods: Optional[list[str]]
    ultimatumMods: Optional[list[UltimatumMod]]
    explicitMods: Optional[list[str]]
    craftedMods: Optional[list[str]]
    fracturedMods: Optional[list[str]]
    crucibleMods: Optional[list[str]]
    cosmeticMods: Optional[list[str]]
    veiledMods: Optional[list[str]]
    veiled: Optional[bool]
    descrText: Optional[str]
    flavourText: Optional[list[str]]
    flavourTextParsed: Optional[list[Union[str, dict]]]
    flavourTextNote: Optional[str]
    prophecyText: Optional[str]
    isRelic: Optional[bool]
    foilVariation: Optional[int]
    replica: Optional[bool]
    foreseeing: Optional[bool]
    incubatedItem: Optional[IncubatedItem]
    scourged: Optional[Scourged]
    crucible: Optional[Crucible]
    ruthless: Optional[bool]
    frameType: Optional[int]
    artFilename: Optional[str]
    hybrid: Optional[Hybrid]
    extended: Optional[Extended]
    x: Optional[int]
    y: Optional[int]
    inventoryId: Optional[str]
    socket: Optional[int]
    colour: Optional[str]


class PublicStashChange(TypedDict):
    id: str
    public: bool
    accountName: Optional[str]
    stash: Optional[str]
    lastCharacterName: Optional[str]
    stashType: str
    league: Optional[str]
    items: list[Item]


class CrucibleNode(TypedDict):
    skill: Optional[int]
    tier: Optional[int]
    icon: Optional[str]
    allocated: Optional[bool]
    isNotable: Optional[bool]
    isReward: Optional[bool]
    stats: Optional[list[str]]
    reminderText: Optional[list[str]]
    orbit: Optional[int]
    orbitIndex: Optional[int]
    out: Optional[list[str]]
    in_: Optional[list[str]]


class ExpansionJewel(TypedDict):
    size: Optional[int]
    index: Optional[int]
    proxy: Optional[int]
    parent: Optional[int]


class MasteryEffect(TypedDict):
    effect: int
    stats: list[str]
    reminderText: Optional[list[str]]


class PassiveNode(TypedDict):
    skill: Optional[int]
    name: Optional[str]
    icon: Optional[str]
    isKeystone: Optional[bool]
    isNotable: Optional[bool]
    isMastery: Optional[bool]
    inactiveIcon: Optional[str]
    activeIcon: Optional[str]
    activeEffectImage: Optional[str]
    masteryEffects: Optional[MasteryEffect]
    isBlighted: Optional[bool]
    isTattoo: Optional[bool]
    isProxy: Optional[bool]
    isJewelSocket: Optional[bool]
    expansionJewel: Optional[ExpansionJewel]
    recipe: Optional[list[str]]
    grantedStrength: Optional[int]
    grantedDexterity: Optional[int]
    grantedIntelligence: Optional[int]
    ascendancyName: Optional[str]
    isAscendancyStart: Optional[bool]
    isMultipleChoice: Optional[bool]
    isMultipleChoiceOption: Optional[bool]
    grantedPassivePoints: Optional[int]
    stats: Optional[list[str]]
    reminderText: Optional[list[str]]
    flavourText: Optional[list[str]]
    classStartIndex: Optional[int]
    group: Optional[str]
    orbit: Optional[int]
    orbitIndex: Optional[int]
    out: Optional[list[str]]
    in_: Optional[list[str]]


class PassiveGroup(TypedDict):
    x: float
    y: float
    orbits: list[int]
    isProxy: Optional[bool]
    proxy: Optional[str]
    nodes: list[str]


class Subgraph(TypedDict):
    groups: dict[str, PassiveGroup]
    nodes: dict[str, PassiveNode]


class ItemJewelData(TypedDict):
    type: str
    radius: Optional[int]
    radiusMin: Optional[int]
    radiusVisual: Optional[str]
    subgraph: Optional[Subgraph]


class CharacterMetadata(TypedDict):
    version: Optional[str]


class Passives(TypedDict):
    hashes: list[int]
    hashes_ex: list[int]
    mastery_effects: dict[str, int]
    skill_overrides: Optional[dict[str, 'PassiveNode']]
    bandit_choice: Optional[str]
    pantheon_major: Optional[str]
    pantheon_minor: Optional[str]
    jewel_data: dict[str, ItemJewelData]
    alternate_ascendancy: Optional[str]


class Character(TypedDict):
    id: str
    name: str
    realm: str
    class_: str
    league: Optional[str]
    level: int
    experience: int
    ruthless: Optional[bool]
    expired: Optional[bool]
    deleted: Optional[bool]
    current: Optional[bool]
    equipment: Optional[list[Item]]
    inventory: Optional[list[Item]]
    rucksack: Optional[list[Item]]
    jewels: Optional[list[Item]]
    passives: Optional[Passives]
    metadata: Optional[CharacterMetadata]


class StashTabMetaDict(TypedDict):
    public: Optional[bool]
    folder: Optional[bool]
    colour: Optional[str]


class StashTab(TypedDict):
    id: str
    parent: Optional[str]
    name: str
    type: str
    index: Optional[int]
    metadata: StashTabMetaDict
    children: Optional[list['StashTab']]
    items: Optional[list[Item]]


class AtlasTreeInfo(TypedDict):
    name: str
    hashess: list[int]


class AtlasTreeInfoOld(TypedDict):
    hashess: list[int]


class LeagueAccount(TypedDict):
    atlas_passives: Optional[AtlasTreeInfoOld]
    atlas_passive_trees: list[AtlasTreeInfo]


class ValidationDetails(TypedDict):
    valid: bool
    version: Optional[str]
    validated: Optional[str]


class ItemFilter(TypedDict):
    id: str
    filter_name: str
    realm: str
    description: str
    version: str
    type: str
    public: Optional[bool]
    filter: Optional[str]
    validation: Optional[ValidationDetails]


class PassiveGroup(TypedDict):
    x: float
    y: float
    orbits: list[int]
    isProxy: Optional[bool]
    proxy: Optional[str]
    nodes: list[str]


class MasteryEffect(TypedDict):
    effect: int
    stats: list[str]
    reminderText: Optional[list[str]]


class ExpansionJewel(TypedDict):
    size: Optional[int]
    index: Optional[int]
    proxy: Optional[int]
    parent: Optional[int]


class PassiveNode(TypedDict):
    skill: Optional[int]
    name: Optional[str]
    icon: Optional[str]
    isKeystone: Optional[bool]
    isNotable: Optional[bool]
    isMastery: Optional[bool]
    inactiveIcon: Optional[str]
    activeIcon: Optional[str]
    activeEffectImage: Optional[str]
    masteryEffects: Optional[list[MasteryEffect]]
    isBlighted: Optional[bool]
    isTattoo: Optional[bool]
    isProxy: Optional[bool]
    isJewelSocket: Optional[bool]
    expansionJewel: Optional[ExpansionJewel]
    recipe: Optional[list[str]]
    grantedStrength: Optional[int]
    grantedDexterity: Optional[int]
    grantedIntelligence: Optional[int]
    ascendancyName: Optional[str]
    isAscendancyStart: Optional[bool]
    isMultipleChoice: Optional[bool]
    isMultipleChoiceOption: Optional[bool]
    grantedPassivePoints: Optional[int]
    stats: Optional[list[str]]
    reminderText: Optional[list[str]]
    flavourText: Optional[list[str]]
    classStartIndex: Optional[int]
    group: Optional[str]
    orbit: Optional[int]
    orbitIndex: Optional[int]
    out: list[str]
    in_: list[str]


class CrucibleNode(TypedDict):
    skill: Optional[int]
    tier: Optional[int]
    icon: Optional[str]
    allocated: Optional[bool]
    isNotable: Optional[bool]
    isReward: Optional[bool]
    stats: Optional[list[str]]
    reminderText: Optional[list[str]]
    orbit: Optional[int]
    orbitIndex: Optional[int]
    out: list[str]
    in_: list[str]



import xml.etree.ElementTree as ET

from schema import Item
from data import gem_data


def get_item_text(item: Item) -> str:
    quality = ""
    sockets = ""
    level_req = ""
    limit = ""
    radius = ""

    for property in item.get("properties", []):
        if property["name"] == "Quality":
            quality = property["values"][0][0].replace(
                "+", "").replace("%", "")
        if property["name"] == "LevelReq":
            level_req = property["values"][0][0]
        if property["name"] == "Limited to":
            limit = property["values"][0][0]
        if property["name"] == "Radius":
            radius = property["values"][0][0]
    socket_group = 0
    sockets = []
    for socket_group in {socket["group"] for socket in item.get("sockets", [])}:
        sockets.append("-".join([socket["sColour"][0]
                                 for socket in item.get("sockets", []) if socket["group"] == socket_group]))
    sockets = " ".join(sockets)

    item_text = []
    item_text.append(f"Rarity: {item['rarity'].upper()}")
    if item["name"]:
        item_text.append(item["name"])
    if item["typeLine"]:
        item_text.append(item["typeLine"])
    item_text.append(f"Unique ID: {item['id']}")
    item_text.append(f"Item Level: {item['ilvl']}")
    if quality:
        item_text.append(f"Quality: {quality}")
    if sockets:
        item_text.append(f"Sockets: {sockets}")
    if level_req:
        item_text.append(f"LevelReq: {level_req}")
    if limit:
        item_text.append(f"Limited to: {limit}")
    if radius:
        item_text.append(f"Radius: {radius}")
    item_text.append(
        f"Implicits: {len(item.get('implicitMods', []) + item.get('enchantMods', []))}")
    item_text.extend(item.get("implicitMods", []))
    item_text.extend(
        ["{crafted}" + mod for mod in item.get("enchantMods", [])])
    item_text.extend(
        ["{fractured}" + mod for mod in item.get("fracturedMods", [])])
    item_text.extend(item.get("explicitMods", []))
    item_text.extend(
        ["{crafted}" + mod for mod in item.get("craftedMods", [])])
    return "\n".join(item_text)


def set_gems(item: Item, skillset: ET.Element) -> None:
    sockets: list[list[int]] = []
    for socket_group in {socket["group"] for socket in item.get("sockets", [])}:
        group_ids = []
        for idx, socket in enumerate(item.get("sockets", [])):
            if socket["group"] == socket_group:
                group_ids.append(idx)
        sockets.append(group_ids)

    for socket_group in sockets:
        skill = ET.SubElement(skillset, "Skill", {
            "mainActiveSkillCalcs": "nil",
            "includeInFullDPS": "nil",
            "label": "",
            "enabled": "true",
            "slot": get_slot(item)
        })
        for socket_id in socket_group:
            if len(item.get("socketedItems", [])) > socket_id:
                gem = item["socketedItems"][socket_id]
                if "abyssJewel" in gem:
                    continue
                ET.SubElement(skill, "Gem", get_gem_stats(gem))


def get_gem_stats(gem: Item) -> dict:
    level = "1"
    quality = "0"
    for property in gem.get("properties", []):
        if property["name"] == "Quality":
            quality = property["values"][0][0].replace(
                "+", "").replace("%", "")
        if property["name"] == "Level":
            level = property["values"][0][0].replace(" (Max)", "")
    gem_info = gem_data.get(gem["baseType"], {})

    gemId = gem_info.get("gemId")
    variantId = gem_info.get("variantId")
    return {
        "level": level,
        "quality": quality,
        "gemId": gemId,
        "variantId": variantId,
        "qualityId": "Default",
        "enabled": "true",
        "count": "nil",
        "nameSpec": gem["baseType"]
    }


def get_slot(item: Item) -> str:
    if item["inventoryId"] == "Flask":
        return f"Flask {item['x'] + 1}"
    return {
        "Weapon": "Weapon 1",
        "Offhand": "Weapon 2",
        "BodyArmour": "Body Armour",
        "Helm": "Helmet",
        "Gloves": "Gloves",
        "Boots": "Boots",
        "Ring": "Ring 1",
        "Ring2": "Ring 2",
        "Amulet": "Amulet",
        "Belt": "Belt",
    }.get(item["inventoryId"], item["inventoryId"])


def get_slots():
    return {
        "Gloves": "0",
        "Weapon 1": "0",
        "Boots Abyssal Socket 5": "0",
        "Belt Abyssal Socket 2": "0",
        "Flask 3": "0",
        "Weapon 1 Swap Abyssal Socket 4": "0",
        "Weapon 1 Abyssal Socket 5": "0",
        "Weapon 2 Abyssal Socket 5": "0",
        "Weapon 1 Swap Abyssal Socket 6": "0",
        "Weapon 1 Abyssal Socket 6": "0",
        "Weapon 2 Abyssal Socket 6": "0",
        "Weapon 2 Swap Abyssal Socket 1": "0",
        "Boots Abyssal Socket 3": "0",
        "Weapon 1 Abyssal Socket 2": "0",
        "Weapon 2 Swap Abyssal Socket 4": "0",
        "Helmet Abyssal Socket 1": "0",
        "Belt Abyssal Socket 6": "0",
        "Helmet Abyssal Socket 2": "0",
        "Helmet": "0",
        "Helmet Abyssal Socket 3": "0",
        "Weapon 2 Swap Abyssal Socket 5": "0",
        "Weapon 1 Abyssal Socket 1": "0",
        "Weapon 2 Abyssal Socket 1": "0",
        "Weapon 1 Abyssal Socket 4": "0",
        "Belt": "0",
        "Weapon 1 Swap Abyssal Socket 2": "0",
        "Boots Abyssal Socket 4": "0",
        "Weapon 2 Abyssal Socket 2": "0",
        "Flask 5": "0",
        "Weapon 1 Swap Abyssal Socket 3": "0",
        "Boots Abyssal Socket 2": "0",
        "Belt Abyssal Socket 3": "0",
        "Body Armour Abyssal Socket 1": "0",
        "Gloves Abyssal Socket 1": "0",
        "Helmet Abyssal Socket 5": "0",
        "Helmet Abyssal Socket 6": "0",
        "Weapon 1 Swap Abyssal Socket 1": "0",
        "Weapon 2 Swap Abyssal Socket 2": "0",
        "Flask 1": "0",
        "Helmet Abyssal Socket 4": "0",
        "Gloves Abyssal Socket 5": "0",
        "Body Armour Abyssal Socket 2": "0",
        "Amulet": "0",
        "Weapon 1 Abyssal Socket 3": "0",
        "Weapon 2 Abyssal Socket 3": "0",
        "Belt Abyssal Socket 4": "0",
        "Belt Abyssal Socket 1": "0",
        "Body Armour Abyssal Socket 3": "0",
        "Weapon 2 Swap": "0",
        "Flask 4": "0",
        "Flask 2": "0",
        "Weapon 2": "0",
        "Belt Abyssal Socket 5": "0",
        "Boots Abyssal Socket 1": "0",
        "Body Armour Abyssal Socket 4": "0",
        "Weapon 2 Swap Abyssal Socket 6": "0",
        "Weapon 2 Swap Abyssal Socket 3": "0",
        "Weapon 1 Swap": "0",
        "Ring 2": "0",
        "Gloves Abyssal Socket 2": "0",
        "Body Armour Abyssal Socket 6": "0",
        "Weapon 1 Swap Abyssal Socket 5": "0",
        "Weapon 2 Abyssal Socket 4": "0",
        "Body Armour": "0",
        "Ring 1": "0",
        "Gloves Abyssal Socket 3": "0",
        "Body Armour Abyssal Socket 5": "0",
        "Gloves Abyssal Socket 4": "0",
        "Gloves Abyssal Socket 6": "0",
        "Boots": "0",
        "Boots Abyssal Socket 6": "0"
    }
